import string
from sgmllib import SGMLParser

def join_attrs(attrs):
   attr_list = []
   for attrname, value in attrs:
      attr_list.append('%s="%s"' % (attrname, string.strip(value)))

   if attr_list:
      s = " " + string.join(attr_list, " ")
   else:
      s = ""

   return s


class HeadParser(SGMLParser):

   from htmlentitydefs import entitydefs # replace entitydefs from sgmllib

   def __init__(self, removejs=0):
      SGMLParser.__init__(self)
      
      self.removejs = removejs
      
      self.seen_starthead = 0
      self.seen_endhead   = 0
      self.seen_startbody = 0

      self.head = ""
      self.title = ""
      self.bodyattribs = ""
      self.accumulator = ""


   def handle_data(self, data):
      if data:
         self.accumulator = self.accumulator + data

   def handle_comment(self, data):
      if data:
         self.accumulator = self.accumulator + "<!--%s-->" % data


   def handle_charref(self, name):
      self.accumulator = "%s&#%s;" % (self.accumulator, name)

   def handle_entityref(self, name):
      if self.entitydefs.has_key(name): # If it is one of the standard SGML entities - close it with semicolon
         x = ';'
      else:
         # this breaks unstandard entities that end with ';'
         x = ''
      self.accumulator = "%s&%s%s" % (self.accumulator, name, x)


   def start_head(self, attrs):
      if not self.seen_starthead:
         self.seen_starthead = 1
         self.head = ""
         self.title = ""
         self.accumulator = ""

   def end_head(self):
      if not self.seen_endhead:
         self.seen_endhead = 1
         self.head = self.head + self.accumulator
         self.accumulator = ""

   def start_script(self, attrs):
       if self.removejs and not self.seen_endhead:
           self.head = self.head + self.accumulator
           self.accumulator = ""
       else:
          self.unknown_starttag('script', attrs)

   def end_script(self):
       if (not self.removejs) or self.seen_endhead:
           self.unknown_endtag('script')
       else:
           self.accumulator = ""

   def start_title(self, attrs):
      self.head = self.head + self.accumulator
      self.accumulator = ""

   def end_title(self):
      self.title = self.accumulator
      self.accumulator = ""


   def start_body(self, attrs):
      if not self.seen_startbody:
         self.seen_startbody = 1
         self.accumulator = ""
         self.bodyattribs = join_attrs(attrs)

   def end_body(self): pass # Do not put </BODY> and </HTML>
   def end_html(self): pass # into output stream


   # Pass other tags unmodified
   def unknown_starttag(self, tag, attrs):
      self.accumulator = self.accumulator + "<%s%s>" % (string.upper(tag), join_attrs(attrs))

   def unknown_endtag(self, tag):
      self.accumulator = self.accumulator + "</%s>" % string.upper(tag)


def clean_html(text):
    #translate Windows-generated html to correct html
    #mainly the ' appearing as ? problem.
    #see http://www.hut.fi/u/jkorpela/www/windows-chars.html
    charmap = string.maketrans("\x91\x92\x93\x94\x96\x97", "''\"\"--") 
    text = string.translate(text, charmap)
    return text


def parse_html(infile, removejs):
   parser = HeadParser(removejs)

   while 1:
      line = infile.readline()
      if not line: break
      parser.feed(line)

   parser.close()
   infile.close()

   return string.strip(parser.title), string.strip(parser.head), parser.bodyattribs, \
"""<dtml-var standard_html_header>

""" + string.strip(parser.accumulator) + """

<dtml-var standard_html_footer>"""
