#!/usr/bin/env python
from wxPython.wx import *
import urllib, urlparse
from wxLoadSite import xmlrpclib
from wxLoadSite.ZopeListControl import ZopeListCtrl
from wxLoadSite.FileListControl import FileListCtrl
from LoadSite import Uploader, Client

class ConnectionDialog(wxDialog):
    """
    Zope Connection Dialog
    """
    def __init__(self, parent, id, title):
        wxDialog.__init__(self, parent, -1, title, size=(300, 170))
        wxStaticText(self, -1, "URL:", (10, 10))
        self.url=wxTextCtrl(self, -1, "http://zope:8080", (70, 10), (200, 20))
        wxStaticText(self, -1, "User name:", (10, 35))
        self.username=wxTextCtrl(self, -1, "superuser", (70, 35), (100, 20))
        wxStaticText(self, -1, "Password:", (10, 60))
        self.password=wxTextCtrl(self, -1, "spoon", (70, 60), (100, 20), wxTE_PASSWORD)
        wxButton(self, wxID_OK, " OK ", (50, 100)).SetDefault()
        wxButton(self, wxID_CANCEL, " Cancel ", (175, 100))

class MainFrame(wxFrame):
    def __init__(self, parent, ID, title):
        wxFrame.__init__(self, parent, ID, title,
                         wxDefaultPosition, wxSize(450, 400))
        self.SetAutoLayout(TRUE)                 
        # --
        self.uploader = Uploader.Uploader()
        
        self.CreateStatusBar()
        self.SetStatusText("Welcome to LoadSite")
        
        # create checkboxes for options
        pID = NewId()
        toppanel = wxPanel(self, pID, wxPoint(0,0), wxSize(450, 50))
        pc = wxLayoutConstraints()
        pc.top.Absolute(0)
        pc.left.Absolute(0)
        pc.bottom.Absolute(50)
        pc.right.SameAs(self, wxRight, 0)
        toppanel.SetConstraints(pc)
        
        cID = NewId()
        cb1 = wxCheckBox(toppanel, cID, "  Strip Header and Footer", wxPoint(10, 5), 
                         wxSize(200, 20), wxNO_BORDER)
        cb2 = wxCheckBox(toppanel, cID + 1, "  Strip JS from Header", wxPoint(10, 25), 
                         wxSize(200, 20), wxNO_BORDER)
        cb3 = wxCheckBox(toppanel, cID + 2, "  Add index_html", wxPoint(220, 5), 
                         wxSize(200, 20), wxNO_BORDER)
        cb4 = wxCheckBox(toppanel, cID + 3, "  Clean HTML", wxPoint(220, 25), 
                         wxSize(200, 20), wxNO_BORDER)
        EVT_CHECKBOX(self, cID, self.EvtCheckBoxDoctor)
        EVT_CHECKBOX(self, cID + 1, self.EvtCheckBoxRemovejs)
        EVT_CHECKBOX(self, cID + 2, self.EvtCheckBoxIndex)
        EVT_CHECKBOX(self, cID + 3, self.EvtCheckBoxClean)
                             
        # create split upload interface
        splitter = wxSplitterWindow(self, -1, wxPoint(0, 50))
        lc = wxLayoutConstraints()
        lc.top.Absolute(50)
        lc.left.Absolute(0)
        lc.bottom.SameAs(self, wxBottom, 10)
        lc.right.SameAs(self, wxRight, 0)
        splitter.SetConstraints(lc)
        
        # Make panel for local file list
        lID = NewId()
        self.panel = wxPanel(splitter, lID)
        bs = wxBoxSizer(wxHORIZONTAL)
        
        lID = NewId()
        self.filelist = FileListCtrl(self.panel, lID)
        bs.Add(self.filelist, 1, wxEXPAND, 0)
        
        lID = NewId()
        wxImage_AddHandler(wxPNGHandler())
        arrow = wxImage('wxLoadSite/images/arrow.png', 
                        wxBITMAP_TYPE_PNG).ConvertToBitmap()
        self.copyButton = wxBitmapButton(self.panel, lID, arrow, 
                                    wxDefaultPosition, wxSize(30, 23))
        EVT_BUTTON(self, lID, self.onPressCopy)                             
        bs.Add(self.copyButton, 0, wxALIGN_CENTER | wxALL, 2)
        
        self.panel.SetAutoLayout(TRUE)
        self.panel.SetSizer(bs)
        bs.Fit(self.panel)
        bs.SetSizeHints(self.panel)
        
        # Make zope objects list
        lID = NewId()
        self.zopelist =  ZopeListCtrl(splitter, lID)
        
        # Arrange windows
        splitter.SplitVertically(self.panel, self.zopelist)
        splitter.SetSashPosition(180, true)
        splitter.SetMinimumPaneSize(20)
        
        # Make connection dialog
        self.connect=ConnectionDialog(self, -1, "Connect to Zope")
        
        # Make File menu
        menu = wxMenu()
        mID = NewId()
        menu.Append(mID, '&Connect', 'Connect to Zope')
        EVT_MENU(self, mID, self.onFileConnect)
        mID = NewId()
        menu.Append(mID, 'E&xit', 'Quit this program')
        EVT_MENU(self, mID, self.onFileExit)

        menuBar = wxMenuBar()
        menuBar.Append(menu, "&File");
        self.SetMenuBar(menuBar)
       
    def onFileExit(self, event):
        self.Close(true)
        
    def onFileConnect(self, event):
        val = self.connect.ShowModal()
        if val == wxID_OK:
            username = self.connect.username.GetValue()
            password = self.connect.password.GetValue()
            url = self.connect.url.GetValue()
            self.zopelist.connectZope(url, username, password)
    
    def onPressCopy(self, event):
        url = 'http://%s/%s/' % (self.zopelist.host, self.zopelist.path)
        self.SetStatusText("Connecting to %s..." % url)
        remotefolder = Client.Object(url, username=self.zopelist.username, 
                                     password=self.zopelist.password)
        for fileItem in self.filelist.selectedItems:
            self.Upload(remotefolder, self.filelist.GetItemText(fileItem))
            
    def Upload(self, remotefolder, filename):
        self.SetStatusText("Uploading %s..." % filename)        
        self.uploader.upload_file(remotefolder, filename)
        self.SetStatusText("")
        self.zopelist.updateList()
    
    def EvtCheckBoxDoctor(self, event):
        self.uploader.setOptions(doctor=event.Checked())

    def EvtCheckBoxRemovejs(self, event):
        self.uploader.setOptions(removejs=event.Checked())
        
    def EvtCheckBoxIndex(self, event):
        self.uploader.setOptions(index_html=event.Checked())
        
    def EvtCheckBoxClean(self, event):
        self.uploader.setOptions(clean=event.Checked())
                   
class LoadSiteApp(wxApp):
    def OnInit(self):
        frame = MainFrame(NULL, -1, "LoadSite")
        frame.Show(true)
        self.SetTopWindow(frame)
        return true

app = LoadSiteApp(0)
app.MainLoop()
