from wxPython.wx import *
from lsListControl import lsListCtrl
import xmlrpclib, urllib, urlparse

class ZopeListCtrl(lsListCtrl):
    def __init__(self, parent, lID):
        lsListCtrl.__init__(self, parent, lID)                                      
        tID = NewId()
        self.locationText = wxTextCtrl(self, tID, "", wxDefaultPosition,
                                       wxDefaultSize, wxTE_PROCESS_ENTER)
        tc = wxLayoutConstraints()
        tc.top.SameAs(self, wxTop, 0)
        tc.left.SameAs(self, wxLeft, 0)
        tc.bottom.Absolute(20)
        tc.right.SameAs(self, wxRight, 0)
        self.locationText.SetConstraints(tc)
        EVT_TEXT_ENTER(self, tID, self.LocationTextEnter)
        
        self.InsertColumn(0, "Name")
        self.InsertColumn(1, "Title")
        self.InsertColumn(2, "Meta Type")
        self.InsertColumn(3, "")
        
        # need to break encapsulation.  oh well
        EVT_RIGHT_DOWN(self._listCtrl, self.onRightDown)
        # for wxMSW
        EVT_COMMAND_RIGHT_CLICK(self._listCtrl, lID, self.onRightClick)
        # for wxGTK
        EVT_RIGHT_UP(self._listCtrl, self.onRightClick)
        
    def connectZope(self, url, username, password):
        """ Connect to Zope server """
        zopeobj = xmlrpclib.Server(url, username=username, password=password)
        try:
            tmp = zopeobj.objectIds()
        except:
            pass
        else:
            parsedurl = urlparse.urlparse(url)
            self.host = parsedurl[1]
            self.path = parsedurl[2]
            self.username = username
            self.password = password
            self.zopeobj = zopeobj
            self.serverobj = zopeobj
            self.updateList()
        
    def updateList(self):
        lsListCtrl.updateList(self)
        self.locationText.Clear()
        self.locationText.WriteText(self.path or '/')
        wxSetCursor(wxStockCursor(wxCURSOR_WAIT))
        self.DeleteAllItems()        
        for obj in self.zopeobj.objectMap():
            id = obj['id']
            meta = obj['meta_type']
            
            i = self.InsertItem(wxListItem())
            
            if meta in ['Folder', 'ZCatalog']: 
                self.SetStringItem(i, 0, id, 0)
            else:  
                self.SetStringItem(i, 0, id, 1)
            self.SetStringItem(i, 2, meta)
                      
            try: self.SetStringItem(i, 1, self.zopeobj[id].getProperty('title'))
            except: pass
            
            self.SetItemText(i, id)
        
        i = self.InsertItem(wxListItem())
        self.SetStringItem(i, 0, '..')
        for j in range(i):
            self.EnsureVisible(j)
        wxSetCursor(wxStockCursor(wxCURSOR_ARROW))    
    
    def changedir(self, dirname):
        if dirname in self.zopeobj.objectIds():
            newobj = getattr(self.zopeobj, dirname)
            # test that this is a folderish object (FIXME)
            try: temp = newobj.objectIds()
            except: return 0
            self.zopeobj = newobj
            self.path = "%s/%s" % (self.path, dirname)
            return 1
        elif dirname == '..':
            jn = string.join
            sp = string.split
            # truncate last item in path
            newpath = jn(sp(self.path, '/')[:-1], '/')
            self.path = newpath
            self.zopeobj = xmlrpclib.Server('http://%s/%s/' % (self.host, self.path),
                                     username=self.username, password=self.password)
            
            return 1
        else:
            return 0
    
    def onRightDown(self, event):
        self.x = event.GetX()
        self.y = event.GetY() + 40
        event.Skip()
        
    def onRightClick(self, event):
        menu = wxMenu()
        tPopupID1 = 0
        tPopupID2 = 1
        menu.Append(tPopupID1, "Refresh")
        menu.AppendSeparator()
        menu.Append(tPopupID2, "Delete Selected")
        EVT_MENU(self, tPopupID1, self.menuRefresh)
        EVT_MENU(self, tPopupID2, self.menuDeleteSelected)
        self.PopupMenu(menu, wxPoint(self.x, self.y))
        menu.Destroy()
    
    def menuRefresh(self, event):
        self.updateList()
        
    def menuDeleteSelected(self, event):
        itemNames = map(self.GetItemText, self.selectedItems)
        self.zopeobj.manage_delObjects(itemNames)
        self.updateList()
        
    def LocationTextEnter(self, event):
        if not hasattr(self, 'zopeobj'): return None
        newpath = self.locationText.GetLineText(0)
        if newpath[-1] == '/': newpath = newpath[:-1]        
        url = "http://%s%s" % (self.host, newpath)
        self.connectZope(url, self.username, self.password)
